@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.3.1
+ *  CU sudo version 1.4
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,20 +15,21 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Please send bugs, changes, problems to sudo-bugs.cs.colorado.edu
+ *  Please send bugs, changes, problems to sudo-bugs@cs.colorado.edu
  *
  *******************************************************************
  *
  *  This module contains the find_path() function that returns
- *  a pointer to a static area with the absolute path of the 
- *  command or NULL if the command is not found in the path.
+ *  TRUE if the command was found and FALSE if not.
+ *  If find_path() returns TRUE, the copyin paramters command and
+ *  ocommand contain the resolved and unresolved pathnames respectively.
  *  NOTE: if "." or "" exists in PATH it will be searched last.
  *
- *  Todd C. Miller (millert@colorado.edu) Sat Sep  4 12:22:04 MDT 1993
+ *  Todd C. Miller (millert@colorado.edu) Sat Mar 25 21:50:36 MST 1995
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: find_path.c,v 1.37 1994/08/12 01:58:03 millert Exp $";
+static char rcsid[] = "$Id: find_path.c,v 1.56 1996/01/14 20:19:22 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
@@ -46,15 +47,16 @@ static char rcsid[] = "$Id: find_path.c,v 1.37 1994/08/12 01:58:03 millert Exp $
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
-#ifdef HAVE_MALLOC_H
+#if defined(HAVE_MALLOC_H) && !defined(STDC_HEADERS)
 #include <malloc.h>
-#endif /* HAVE_MALLOC_H */
+#endif /* HAVE_MALLOC_H && !STDC_HEADERS */
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <netinet/in.h>
 #include "sudo.h"
+#include <options.h>
 
 #ifndef STDC_HEADERS
 #ifndef __GNUC__		/* gcc has its own malloc */
@@ -63,7 +65,7 @@ extern char *malloc	__P((size_t));
 extern char *getenv	__P((const char *));
 extern char *strcpy	__P((char *, const char *));
 extern int fprintf	__P((FILE *, const char *, ...));
-extern int readlink	__P((const char *, char *, size_t));
+extern ssize_t readlink	__P((const char *, VOID *, size_t));
 extern int stat		__P((const char *, struct stat *));
 extern int lstat	__P((const char *, struct stat *));
 #ifdef HAVE_STRDUP
@@ -80,62 +82,63 @@ extern char *strdup	__P((const char *));
 #endif /* _S_IFLNK */
 
 
-/*
- * Globals
- */
-static char * realpath_exec	__P((char *, char *, char *));
-
-
 /*******************************************************************
  *
  *  find_path()
  *
- *  this function finds the full pathname for a command
+ *  this function finds the full pathname for a command and
+ *  stores it in a statically allocated array, returning a pointer
+ *  to the array.
  */
 
-char *find_path(file)
-    char *file;
+char * find_path(file)
+    char *file;			/* file to find */
 {
+    static char command[MAXPATHLEN + 1];	/* qualified filename */
     register char *n;		/* for traversing path */
     char *path = NULL;		/* contents of PATH env var */
-    char *oldpath;		/* so we can free path later */
+    char *origpath;		/* so we can free path later */
     char *result = NULL;	/* result of path/file lookup */
-    char *command;		/* resolved pathname */
     int checkdot = 0;		/* check current dir? */
 
+    command[0] = '\0';
+
     if (strlen(file) > MAXPATHLEN) {
+	errno = ENAMETOOLONG;
 	(void) fprintf(stderr, "%s:  path too long:  %s\n", Argv[0], file);
 	exit(1);
     }
 
     /*
-     * allocate memory for command
+     * If we were given a fully qualified or relative path
+     * there is no need to look at PATH.
+     * We really want to fall back if !sudo_goodpath() but then
+     * the error is "not found" -- this way we get the correct error.
      */
-    command = (char *) malloc(MAXPATHLEN + 1);
-    if (command == NULL) {
-	perror("malloc");
-	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
-	exit(1);
+    if (strchr(file, '/')) {
+	(void) strcpy(command, file);
+	if (sudo_goodpath(command)) {
+	    return(command);
+	} else {
+	    (void) fprintf(stderr, "%s: %s: ", Argv[0], command);
+	    perror("");
+	    exit(1);
+	}
     }
 
-    /*
-     * do we need to search the path?
-     */
-    if (strchr(file, '/'))
-	return((char *) sudo_realpath(file, command));
-
     /*
      * grab PATH out of environment and make a local copy
      */
     if ((path = getenv("PATH")) == NULL)
-	return (NULL);
+	return(NULL);
 
-    if ((path = strdup(path)) == NULL) {
-	fprintf(stderr, "sudo: out of memory!\n");
+    if ((path = (char *) strdup(path)) == NULL) {
+	(void) fprintf(stderr, "%s: out of memory!\n", Argv[0]);
 	exit(1);
     }
-    oldpath=path;
+    origpath=path;
 
+    /* XXX use strtok() */
     do {
 	if ((n = strchr(path, ':')))
 	    *n = '\0';
@@ -153,7 +156,13 @@ char *find_path(file)
 	/*
 	 * resolve the path and exit the loop if found
 	 */
-	if ((result = realpath_exec(path, file, command)))
+	if (strlen(path) + strlen(file) >= MAXPATHLEN) {
+	    errno = ENAMETOOLONG;
+	    (void) fprintf(stderr, "%s:  path too long:  %s\n", Argv[0], file);
+	    exit(1);
+	}
+	(void) sprintf(command, "%s/%s", path, file);
+	if ((result = sudo_goodpath(command)))
 	    break;
 
 	path = n + 1;
@@ -163,45 +172,12 @@ char *find_path(file)
     /*
      * check current dir if dot was in the PATH
      */
-    if (!result && checkdot)
-	result = realpath_exec(".", file, command);
-
-    (void) free(oldpath);
-    return (result);
-}
-
-
-/*******************************************************************
- *
- *  realpath_exec()
- *
- *  This function calls realpath() to resolve the path and checks
- *  so see that file is executable.  Returns the resolved path on
- *  success and NULL on failure (or if file is not executable).
- */
-
-static char * realpath_exec(path, file, command)
-    char * path;
-    char * file;
-    char * command;
-{
-    char fn[MAXPATHLEN+1];		/* filename (path + file) */
-    struct stat statbuf;		/* for stat(2) */
-
-    (void) sprintf(fn, "%s/%s", path, file);
-
-    /* resolve the path */
-    errno = 0;
-    if (sudo_realpath(fn, command)) {
-	/* stat the file to make sure it is executable */
-	if (stat(command, &statbuf) == 0 && (statbuf.st_mode & 0000111))
-	    return(command);
-    } else if (errno && errno != ENOENT && errno != ENOTDIR && errno != EINVAL
-	&& errno != EPERM && errno != EACCES) {
-	/* sudo_realpath() got an error */
-	fprintf(stderr, "sudo: Error resolving %s: ", fn);
-	perror("");
+    if (!result && checkdot) {
+	path = ".";
+	result = sudo_goodpath(command);
     }
 
-    return(NULL);
+    (void) free(origpath);
+
+    return(result);
 }
