@@ -1,9 +1,9 @@
 /*
- * CU sudo version 1.3.1 (based on Root Group sudo version 1.1)
+ * CU sudo version 1.4 (based on Root Group sudo version 1.1)
  *
  * This software comes with no waranty whatsoever, use at your own risk.
  *
- * Please send bugs, changes, problems to sudo-bugs.cs.colorado.edu
+ * Please send bugs, changes, problems to sudo-bugs@cs.colorado.edu
  *
  */
 
@@ -37,12 +37,15 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: logging.c,v 1.28 1994/08/12 01:58:58 millert Exp $";
+static char rcsid[] = "$Id: logging.c,v 1.69 1996/01/14 20:19:56 millert Exp $";
 #endif /* lint */
 
 #include "config.h"
 
 #include <stdio.h>
+#ifdef STDC_HEADERS
+#include <stdlib.h>
+#endif /* STDC_HEADERS */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
@@ -52,15 +55,21 @@ static char rcsid[] = "$Id: logging.c,v 1.28 1994/08/12 01:58:58 millert Exp $";
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif /* HAVE_STRINGS_H */
+#if defined(HAVE_MALLOC_H) && !defined(STDC_HEADERS)
+#include <malloc.h>   
+#endif /* HAVE_MALLOC_H && !STDC_HEADERS */
+#include <pwd.h>
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <sys/errno.h>
 #include <netinet/in.h>
 
 #include "sudo.h"
+#include <options.h>
 
 /*
  * Prototypes for local functions
@@ -68,11 +77,46 @@ static char rcsid[] = "$Id: logging.c,v 1.28 1994/08/12 01:58:58 millert Exp $";
 static void send_mail		__P((void));
 static RETSIGTYPE reapchild	__P((int));
 static int appropriate		__P((int));
+#ifdef BROKEN_SYSLOG
+static void syslog_wrapper	__P((int, char *, char *, char *));
+#endif /* BROKEN_SYSLOG */
 
 /*
  * Globals
  */
-static char logline[MAXLOGLEN + 8];
+static char *logline;
+extern int errorlineno;
+
+
+#ifdef BROKEN_SYSLOG
+#define MAXSYSLOGTRIES		16	/* num of retries for broken syslogs */
+#define SYSLOG(a,b,c,d)		syslog_wrapper(a,b,c,d)
+
+/****************************************************************
+ *
+ *  syslog_wrapper()
+ *
+ *  This function logs via syslog w/ a priority and 3 strings args.
+ *  It really shouldn't be necesary but some syslog()'s don't
+ *  guarantee that the syslog() operation will succeed!
+ */
+
+static void syslog_wrapper(pri, fmt, arg1, arg2)
+    int pri;
+    char *fmt;
+    char *arg1;
+    char *arg2;
+{
+    int i;
+
+    for (i = 0; i < MAXSYSLOGTRIES; i++)
+	if (syslog(pri, fmt, arg1, arg2) == 0)
+	    break;
+}
+#else
+#define SYSLOG(a,b,c,d)		syslog(a,b,c,d)
+#endif /* BROKEN_SYSLOG */
+
 
 
 /**********************************************************************
@@ -86,85 +130,88 @@ static char logline[MAXLOGLEN + 8];
 void log_error(code)
     int code;
 {
-    int argc;
-    char **argv;
-    mode_t oldmask;
-    register char *p;
-    register int count;
-#ifdef _PATH_SUDO_LOGFILE
-    register FILE *fp;
+    char *p;
+    int count;
     time_t now;
-#endif /* _PATH_SUDO_LOGFILE */
-#ifdef SYSLOG
-    register int pri;		/* syslog priority */
+#if (LOGGING & SLOG_FILE)
+    mode_t oldmask;
+    FILE *fp;
+#endif /* LOGGING & SLOG_FILE */
+#if (LOGGING & SLOG_SYSLOG)
+    int pri = Syslog_priority_NO;	/* syslog priority, assume the worst */
     char *tmp, save;
-#endif /* SYSLOG */
+#endif /* LOGGING & SLOG_SYSLOG */
 
     /*
-     * there is no need to log the date and time twice if using syslog
+     * Allocate enough memory for logline so we won't overflow it
+     * XXX - don't use 33, use a macro!
+     */
+    count = 33 + 128 + 2 * MAXPATHLEN + strlen(tty) + strlen(cwd);
+    if (cmnd_args)
+	count += strlen(cmnd_args);
+
+    logline = (char *) malloc(count);
+    if (logline == NULL) {
+	perror("malloc");
+	(void) fprintf(stderr, "%s: cannot allocate memory!\n", Argv[0]);
+	exit(1);
+    }
+
+    /*
+     * we will skip this stuff when using syslog(3) but it is
+     * necesary for mail and file logs.
      */
-#ifdef _PATH_SUDO_LOGFILE
     now = time((time_t) 0);
-    (void) sprintf(logline, "%19.19s : %8.8s : ", ctime(&now), user);
-#endif /* _PATH_SUDO_LOGFILE */
+    (void) sprintf(logline, "%19.19s : %8.8s : ", ctime(&now), user_name);
 
     /*
-     * we need a pointer to the end of logline
+     * we need a pointer to the end of logline (XXX - use a #define not 33)
      */
-    p = logline + strlen(logline);
+    p = logline + 33;
 
     switch (code) {
 
 	case ALL_SYSTEMS_GO:
-	    (void) sprintf(p, "PWD=%s ; COMMAND=", cwd);
-#ifdef SYSLOG
+	    (void) sprintf(p, "TTY=%s ; PWD=%s ; COMMAND=", tty, cwd);
+#if (LOGGING & SLOG_SYSLOG)
 	    pri = Syslog_priority_OK;
-#endif /* SYSLOG */
+#endif /* LOGGING & SLOG_SYSLOG */
 	    break;
 
 	case VALIDATE_NO_USER:
-	    (void) sprintf(p, "user NOT in sudoers ; PWD=%s ; COMMAND=", cwd);
-#ifdef SYSLOG
-	    pri = Syslog_priority_NO;
-#endif /* SYSLOG */
+	    (void) sprintf(p,
+		"user NOT in sudoers ; TTY=%s ; PWD=%s ; COMMAND=", tty, cwd);
 	    break;
 
 	case VALIDATE_NOT_OK:
-	    (void) sprintf(p, "command not allowed ; PWD=%s ; COMMAND=", cwd);
-#ifdef SYSLOG
-	    pri = Syslog_priority_NO;
-#endif /* SYSLOG */
+	    (void) sprintf(p,
+		"command not allowed ; TTY=%s ; PWD=%s ; COMMAND=", tty, cwd);
 	    break;
 
 	case VALIDATE_ERROR:
-	    (void) sprintf(p, "error in %s ; PWD=%s ; command: ",
-		_PATH_SUDO_SUDOERS, cwd);
-#ifdef SYSLOG
-	    pri = Syslog_priority_NO;
-#endif /* SYSLOG */
+	    (void) sprintf(p, "error in %s, line %d ; TTY=%s ; PWD=%s.  ",
+		_PATH_SUDO_SUDOERS, errorlineno, tty, cwd);
 	    break;
 
 	case GLOBAL_NO_PW_ENT:
-	    (void) sprintf(p, "There is no /etc/passwd entry for uid %d.  ",
-		uid);
-#ifdef SYSLOG
-	    pri = Syslog_priority_NO;
-#endif /* SYSLOG */
+	    (void) sprintf(p,
+		"There is no passwd entry for uid %ld (TTY=%s).  ",
+		(long) user_uid, tty);
 	    break;
 
 	case PASSWORD_NOT_CORRECT:
-	    (void) sprintf(p, "%d incorrect passwords ; PWD=%s ; COMMAND=",
-		    TRIES_FOR_PASSWORD, cwd);
-#ifdef SYSLOG
-	    pri = Syslog_priority_NO;
-#endif /* SYSLOG */
+	    (void) sprintf(p,
+		"password incorrect ; TTY=%s ; PWD=%s ; COMMAND=", tty, cwd);
+	    break;
+
+	case PASSWORDS_NOT_CORRECT:
+	    (void) sprintf(p,
+		"%d incorrect passwords ; TTY=%s ; PWD=%s ; COMMAND=",
+		    TRIES_FOR_PASSWORD, tty, cwd);
 	    break;
 
 	case GLOBAL_NO_HOSTNAME:
 	    strcat(p, "This machine does not have a hostname ");
-#ifdef SYSLOG
-	    pri = Syslog_priority_NO;
-#endif /* SYSLOG */
 	    break;
 
 	case NO_SUDOERS_FILE:
@@ -174,65 +221,79 @@ void log_error(code)
 			_PATH_SUDO_SUDOERS);
 		    break;
 		case EACCES:
-		    (void) sprintf(p, "%s needs to run setuid root.  ",
-			Argv[0]);
+		    (void) sprintf(p, "Can't read %s.  ", Argv[0]);
 		    break;
 		default:
 		    (void) sprintf(p, "There is a problem opening %s ",
 			_PATH_SUDO_SUDOERS);
 		    break;
 	    }
-#ifdef SYSLOG
-	    pri = Syslog_priority_NO;
-#endif /* SYSLOG */
 	    break;
 
 	case GLOBAL_HOST_UNREGISTERED:
 	    (void) sprintf(p, "gethostbyname() cannot find host %s ", host);
-#ifdef SYSLOG
-	    pri = Syslog_priority_NO;
-#endif /* SYSLOG */
+	    break;
+
+	case SUDOERS_NO_OWNER:
+	    (void) sprintf(p, "no passwd entry for sudoers file owner (%s) ",
+		SUDOERS_OWNER);
+	    break;
+
+	case SUDOERS_NOT_FILE:
+	    (void) sprintf(p, "%s is not a regular file ", _PATH_SUDO_SUDOERS);
+	    break;
+
+	case SUDOERS_WRONG_OWNER:
+	    (void) sprintf(p, "%s is not owned by %s ", _PATH_SUDO_SUDOERS,
+		SUDOERS_OWNER);
+	    break;
+
+	case SUDOERS_RW_OTHER:
+	    (void) sprintf(p, "%s is readable or writeable by other than %s ",
+		_PATH_SUDO_SUDOERS, SUDOERS_OWNER);
+	    break;
+
+	case SPOOF_ATTEMPT:
+	    (void) sprintf(p,
+		"probable spoofing attempt; TTY=%s ; PWD=%s ; COMMAND=",
+		tty, cwd);
+	    break;
+
+	case BAD_STAMPDIR:
+	    (void) sprintf(p,
+	    "%s owned by non-root or not mode 0700; TTY=%s ; PWD=%s ; COMMAND=",
+	    _PATH_SUDO_TIMEDIR, tty, cwd);
+	    break;
+
+	case BAD_STAMPFILE:
+	    (void) sprintf(p,
+		"preposterous stampfile date; TTY=%s ; PWD=%s ; COMMAND=",
+		tty, cwd);
 	    break;
 
 	default:
 	    strcat(p, "found a wierd error : ");
-#ifdef SYSLOG
-	    pri = Syslog_priority_NO;
-#endif /* SYSLOG */
 	    break;
     }
 
 
     /*
-     * if this error is from load_globals() don't put  argv in the message
+     * If this is a parse error or if the error is from load_globals()
+     * don't put  argv in the message.
      */
-    if (!(code & GLOBAL_PROBLEM)) {
+    if (code != VALIDATE_ERROR && !(code & GLOBAL_PROBLEM)) {
 
-	strcat(logline, cmnd);	/* stuff the command into the logline */
+	/* stuff the command into the logline */
+	/* XXX - this could be sped up */
+	strcat(logline, cmnd);
 	strcat(logline, " ");
-
-	argc = Argc - 2;
-	argv = Argv;
-	argv++;
-	p = logline + strlen(logline);
-	count = (int) (logline + MAXLOGLEN - p);
-
-	/*
-	 * Now stuff as much of the rest of the line as will fit
-	 * Do word wrap if logging to a file.
-	 */
-	while (count > 0 && argc--) {
-	    strncpy(p, *(++argv), count);
-	    strcat(p, " ");
-	    p += 1 + (count < strlen(*argv) ? count : strlen(*argv));
-	    count = (int) (logline + MAXLOGLEN - p);
+	if (cmnd_args) {
+	    strcat(logline, cmnd_args);
+	    strcat(logline, " ");
 	}
-	if (count <= 0)		/* if the line is too long, */
-	    strcat(p, " ... ");	/* add an elipsis to the end */
-
     }
 
-#ifdef SYSLOG
+#if (LOGGING & SLOG_SYSLOG)
 #ifdef Syslog_facility
     openlog(Syslog_ident, Syslog_options, Syslog_facility);
 #else
@@ -242,12 +303,13 @@ void log_error(code)
     /*
      * Log the full line, breaking into multiple syslog(3) calls if necesary
      */
-    p = (char *)logline;
-    for (count=0; count < (strlen(logline) / MAXSYSLOGLEN) + 1; count++) {
+    p = &logline[33];			/* skip past the date and user */
+    for (count = 0; count < strlen(logline) / MAXSYSLOGLEN + 1; count++) {
 	if (strlen(p) > MAXSYSLOGLEN) {
 	    /*
 	     * Break up the line into what will fit on one syslog(3) line
 	     * Try to break on a word boundary if possible.
+	     * XXX - speed this up!
 	     */
 	    for (tmp = p + MAXSYSLOGLEN; tmp > p && *tmp != ' '; tmp--)
 		;
@@ -259,9 +321,9 @@ void log_error(code)
 	    *tmp = '\0';
 
 	    if (count == 0)
-		syslog(pri, "%8.8s : %s", user, p);
+		SYSLOG(pri, "%8.8s : %s", user_name, p);
 	    else
-		syslog(pri, "%8.8s : (command continued) %s", user, p);
+		SYSLOG(pri, "%8.8s : (command continued) %s", user_name, p);
 
 	    *tmp = save;			/* restore saved character */
 
@@ -270,17 +332,17 @@ void log_error(code)
 		;
 	} else {
 	    if (count == 0)
-		syslog(pri, "%8.8s : %s", user, p);
+		SYSLOG(pri, "%8.8s : %s", user_name, p);
 	    else
-		syslog(pri, "%8.8s : (command continued) %s", user, p);
+		SYSLOG(pri, "%8.8s : (command continued) %s", user_name, p);
 	}
     }
     closelog();
-#endif /* SYSLOG */
-#ifdef _PATH_SUDO_LOGFILE
+#endif /* LOGGING & SLOG_SYSLOG */
+#if (LOGGING & SLOG_FILE)
 
     /* become root */
-    be_root();
+    set_perms(PERM_ROOT);
 
     oldmask = umask(077);
     fp = fopen(_PATH_SUDO_LOGFILE, "a");
@@ -290,7 +352,7 @@ void log_error(code)
 	send_mail();
     } else {
 	char *beg, *oldend, *end;
-	register int maxlen = MAXLOGFILELEN;
+	int maxlen = MAXLOGFILELEN;
 
 	/*
 	 * Print out logline with word wrap
@@ -344,8 +406,8 @@ void log_error(code)
     }
 
     /* relinquish root */
-    be_user();
-#endif /* _PATH_SUDO_LOGFILE */
+    set_perms(PERM_USER);
+#endif /* LOGGING & SLOG_FILE */
 
     /* send mail if appropriate */
     if (appropriate(code))
@@ -373,18 +435,12 @@ static void send_mail()
     char *mailer = MAILER;
     char *subject = MAILSUBJECT;
     int fd[2];
-    char buf[MAXLOGLEN + 1024];
 #ifdef POSIX_SIGNALS
     struct sigaction action;
 
-    (void) bzero((char *)(&action), sizeof(action));
+    (void) memset((VOID *)&action, 0, sizeof(action));
 #endif /* POSIX_SIGNALS */
 
-    if ((mailer = find_path(mailer)) == NULL) {
-	(void) fprintf(stderr, "%s: mailer (%s) not found\n", Argv[0], mailer);
-	exit(1);
-    }
-
     /* catch children as they die */
 #ifdef POSIX_SIGNALS
     action.sa_handler = reapchild;
@@ -399,7 +455,7 @@ static void send_mail()
     /*
      * we don't want any security problems ...
      */
-    be_full_user();
+    set_perms(PERM_FULL_USER);
     
 #ifdef POSIX_SIGNALS
     action.sa_handler = SIG_IGN;
@@ -432,10 +488,10 @@ static void send_mail()
 	(void) close(0);
 
 	/* feed the data to sendmail */
-	(void) sprintf(buf, "To: %s\nSubject: %s\n\n%s\n\n",
-		ALERTMAIL, subject, logline);
-	write(1, buf, strlen(buf));
-	close(1);
+	/* XXX - do we need to fdopen this fd #1 to a new stream??? */
+	(void) fprintf(stdout, "To: %s\nSubject: %s\n\n%s : %s\n\n",
+		ALERTMAIL, subject, host, logline);
+	fclose(stdout);
 
 	exit(0);
     }
@@ -483,49 +539,109 @@ void inform_user(code)
     switch (code) {
 	case VALIDATE_NO_USER:
 	    (void) fprintf(stderr,
-		    "%s is not in the sudoers file.  This incident will be reported.\n\n",
-		    user);
+		"%s is not in the sudoers file.  This incident will be reported.\n\n",
+		user_name);
 	    break;
 
 	case VALIDATE_NOT_OK:
-	    (void) fprintf(stderr,
-		    "Sorry, user %s is not allowed to execute %s\n\n",
-		    user, cmnd);
+	    if (cmnd_args)
+		(void) fprintf(stderr,
+		    "Sorry, user %s is not allowed to execute \"%s %s\" on %s.\n\n",
+		    user_name, cmnd, cmnd_args, host);
+	    else
+		(void) fprintf(stderr,
+		    "Sorry, user %s is not allowed to execute \"%s\" on %s.\n\n",
+		    user_name, cmnd, host);
 	    break;
 
 	case VALIDATE_ERROR:
 	    (void) fprintf(stderr,
-		    "Sorry, there is a fatal error in the sudoers file.\n\n");
+		"Sorry, there is a fatal error in the sudoers file.\n\n");
 	    break;
 
 	case GLOBAL_NO_PW_ENT:
 	    (void) fprintf(stderr,
-		    "Intruder Alert!  You don\'t exist in the passwd file\n\n");
+		"Intruder Alert!  You don't exist in the passwd file\n\n");
 	    break;
 
-	case GLOBAL_NO_AUTH_ENT:
+	case GLOBAL_NO_SPW_ENT:
 	    (void) fprintf(stderr,
-		    "Intruder Alert!  You don\'t exist in the auth database\n\n");
+		"Intruder Alert!  You don't exist in the shadow passwd file\n\n");
 	    break;
 
 	case GLOBAL_NO_HOSTNAME:
 	    (void) fprintf(stderr,
-		    "This machine does not have a hostname\n\n");
+		"This machine does not have a hostname\n\n");
 	    break;
 
 	case GLOBAL_HOST_UNREGISTERED:
 	    (void) fprintf(stderr,
-		    "This machine is not available via gethostbyname()\n\n");
+		"This machine is not available via gethostbyname()\n\n");
 	    break;
 
 	case PASSWORD_NOT_CORRECT:
+	    (void) fprintf(stderr, "Password not entered correctly\n\n");
+	    break;
+
+	case PASSWORDS_NOT_CORRECT:
 	    (void) fprintf(stderr, "Password not entered correctly after %d tries\n\n",
-		    TRIES_FOR_PASSWORD);
+		TRIES_FOR_PASSWORD);
+	    break;
+
+	case SUDOERS_NO_OWNER:
+	    (void) fprintf(stderr,
+		"No passwd entry for sudoers file owner (%s)\n", SUDOERS_OWNER);
+	    break;
+
+	case NO_SUDOERS_FILE:
+	    switch (errno) {
+		case ENOENT:
+		    (void) fprintf(stderr, "There is no %s file.\n",
+			_PATH_SUDO_SUDOERS);
+		    break;
+		default:
+		    (void) fprintf(stderr, "Can't read %s: ",
+			_PATH_SUDO_SUDOERS);
+		    perror("");
+		    break;
+	    }
+	    break;
+
+	case SUDOERS_NOT_FILE:
+	    (void) fprintf(stderr,
+		"%s is not a regular file!\n", _PATH_SUDO_SUDOERS);
+	    break;
+
+	case SUDOERS_WRONG_OWNER:
+	    (void) fprintf(stderr, "%s is not owned by %s!\n",
+		_PATH_SUDO_SUDOERS, SUDOERS_OWNER);
+	    break;
+
+	case SUDOERS_RW_OTHER:
+	    (void) fprintf(stderr,
+		"%s is readable or writeable by other than %s!\n",
+		_PATH_SUDO_SUDOERS, SUDOERS_OWNER);
+	    break;
+
+	case SPOOF_ATTEMPT:
+	    (void) fprintf(stderr,
+		"%s is not the same command that was validated, disallowing.\n",
+		cmnd);
+	    break;
+
+	case BAD_STAMPDIR:
+	    (void) fprintf(stderr,
+		"Timestamp directory has wrong permissions, ignoring.\n");
+	    break;
+
+	case BAD_STAMPFILE:
+	    (void) fprintf(stderr,
+		"Your timestamp file has a preposterous date, ignoring.\n");
 	    break;
 
 	default:
 	    (void) fprintf(stderr,
-		    "Something wierd happened.\n\n");
+		"Something wierd happened.\n\n");
 	    break;
     }
 }
@@ -550,6 +666,7 @@ static int appropriate(code)
      */
     case VALIDATE_OK:
     case PASSWORD_NOT_CORRECT:
+    case PASSWORDS_NOT_CORRECT:
 /*  case ALL_SYSTEMS_GO:               this is the same as OK */
 	return (0);
 	break;
@@ -575,6 +692,9 @@ static int appropriate(code)
      */
     case VALIDATE_ERROR:
     case NO_SUDOERS_FILE:
+    case SPOOF_ATTEMPT:
+    case BAD_STAMPDIR:
+    case BAD_STAMPFILE:
     default:
 	return (1);
 	break;
