@@ -1,5 +1,5 @@
 /*
- *  CU sudo version 1.3.1
+ *  CU sudo version 1.4
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Please send bugs, changes, problems to sudo-bugs.cs.colorado.edu
+ *  Please send bugs, changes, problems to sudo-bugs@cs.colorado.edu
  *
  *******************************************************************
  *
@@ -26,14 +26,27 @@
  *  Todd C. Miller  Sun Jun  5 17:22:31 MDT 1994
  */
 
+#ifndef lint
+static char rcsid[] = "$Id: tgetpass.c,v 1.33 1996/01/14 20:19:56 millert Exp $";
+#endif /* lint */
+
 #include "config.h"
-#include "pathnames.h"
 
 #include <stdio.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif /* HAVE_STRING_H */
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif /* HAVE_STRINGS_H */
+#include <pwd.h>
 #include <sys/types.h>
+#ifdef HAVE_SYS_BSDTYPES_H
+#include <sys/bsdtypes.h>
+#endif /* HAVE_SYS_BSDTYPES_H */
 #ifdef _AIX
 #include <sys/select.h>
 #endif /* _AIX */
@@ -51,12 +64,15 @@
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
 
+#include <pathnames.h>
+#include "compat.h"
+
 #ifndef _PASSWD_LEN
-#ifdef HAVE_C2_SECURITY
-#define	_PASSWD_LEN	24
-#else
-#define	_PASSWD_LEN	8
-#endif /* HAVE_C2_SECURITY */
+#  ifdef HAVE_C2_SECURITY
+#    define	_PASSWD_LEN	24
+#  else
+#    define	_PASSWD_LEN	8
+#  endif /* HAVE_C2_SECURITY */
 #endif /* _PASSWD_LEN */
 
 
@@ -75,26 +91,24 @@ char * tgetpass(prompt, timeout)
 {
 #ifdef HAVE_TERMIOS_H
     struct termios term;
+    tcflag_t svflagval;
 #else
 #ifdef HAVE_TERMIO_H
     struct termio term;
+    unsigned short svflagval;
 #else
     struct sgttyb ttyb;
+    int svflagval;
 #endif /* HAVE_TERMIO_H */
 #endif /* HAVE_TERMIOS_H */
-    FILE * input, * output;
-    static char buf[_PASSWD_LEN + 1];
 #ifdef POSIX_SIGNALS
     sigset_t oldmask;
     sigset_t mask;
 #else
     int oldmask;
-#endif
-#ifdef HAVE_TERMIO_H
-    tcflag_t svflagval;
-#else
-    int svflagval;
-#endif
+#endif /* POSIX_SIGNALS */
+    FILE * input, * output;
+    static char buf[_PASSWD_LEN + 1];
     fd_set readfds;
     struct timeval tv;
     char *tmp;
@@ -103,7 +117,7 @@ char * tgetpass(prompt, timeout)
      * mask out SIGINT, should probably just catch it.
      */
 #ifdef POSIX_SIGNALS
-    (void) bzero((char *)(&mask), sizeof(mask));
+    (void) memset((VOID *)&mask, 0, sizeof(mask));
     (void) sigaddset(&mask, SIGINT);
     (void) sigprocmask(SIG_BLOCK, &mask, &oldmask);
 #else
@@ -169,13 +183,13 @@ char * tgetpass(prompt, timeout)
     if (select(getdtablesize(), &readfds, NULL, NULL, &tv) <= 0) {
 #endif /* HAVE_SYSCONF */
 	buf[0] = '\0';
-	goto cleanup;
+	goto cleanup;			/* XXX - goto considered harmful */
     }
 
     /* get the password */
     if (!fgets(buf, sizeof(buf), input)) {
 	buf[0] = '\0';
-	goto cleanup;
+	goto cleanup;			/* XXX - goto considered harmful */
     }
 
     if (*(tmp = &buf[strlen(buf)-1]) == '\n')
